import sys
from dataclasses import dataclass
from functools import cached_property
from typing import ClassVar, TypeAlias, TypedDict
import bs4
import httpx
from bs4 import BeautifulSoup


YearStats: TypeAlias = dict[str, int | float]
CarreerStats: TypeAlias = dict[str, YearStats]


class StatCast(TypedDict):
    hitting: CarreerStats
    pitching: CarreerStats


@dataclass
class Player:

    SEP: ClassVar[str] = ", "
    BASE_URL: ClassVar[str] = "https://baseballsavant.mlb.com/savant-player/"

    def __init__(self, id: str, fullname: str) -> None:
        if self.SEP not in fullname:
            raise ValueError("Not a player")
        self.id: int = int(id)
        self.fullname: str = fullname

    @property
    def firstname(self) -> str:
        return self.fullname.split(self.SEP)[-1]

    @property
    def lastname(self) -> str:
        return self.fullname.split(self.SEP)[0]

    @property
    def slug(self) -> str:
        return "-".join([self.firstname, self.lastname, str(self.id)]).lower()

    @property
    def url(self) -> str:
        return f"{self.BASE_URL}{self.slug}"

    @cached_property
    def soup(self) -> BeautifulSoup:
        return BeautifulSoup(self._download_stats(), "html.parser")

    def _download_stats(self) -> bytes:
        resp: httpx.Response = httpx.get(self.url)
        resp.raise_for_status()
        return resp.content

    def __repr__(self) -> str:
        return (
            f'<Player(id={self.id}, lastname="{self.lastname}", '
            f'firstname="{self.firstname}")>'
        )

    @property
    def statcast(self) -> StatCast:
        return {"hitting": self.hitting, "pitching": self.pitching}

    @cached_property
    def pitching(self) -> CarreerStats:
        return {}  # <= TODO

    @cached_property
    def hitting(self) -> CarreerStats:
        stats: CarreerStats = {}
        statcast: bs4.Tag | None = self.soup.select_one(
            "section#statcast-rankings table"
        )
        if statcast:
            stats = self._get_statcast_values(statcast)
        return stats

    def _get_statcast_header(self, statcast: bs4.Tag) -> list[str]:
        return [cell.text.strip() for cell in statcast.select("thead tr th")]

    def _get_statcast_values(self, statcast: bs4.Tag) -> CarreerStats:
        stats: CarreerStats = {}

        def _cast_to_numeric(year_stats: dict[str, str]) -> dict[str, int | float]:
            converted: dict[str, int | float] = {}
            for key, val in year_stats.items():
                if not val:
                    val = 0
                try:
                    converted[key] = int(val)
                except ValueError:
                    converted[key] = float(val)
            return converted

        for line in statcast.select("tbody tr"):
            year_stats: dict[str, str] = {
                self._get_statcast_header(statcast)[cell_id]: cell.text.strip()
                for cell_id, cell in enumerate(line.select("td"))
            }
            year: str = year_stats.pop("Season")
            stats[year] = _cast_to_numeric(year_stats)

        return stats
